% function [f,J,] = dXdT(t,x,T,BX,K_BX,par)
%
%
%
% Output parameters:
%   f     time derivatives of the model
%   J     flux
%
%
% Mandatory input parameters:
%   t     time
%   x     state variables at t=0
%   T     temperature in degreesCelcius
%   BX    buffer sizes
%   K_BX  proton buffer dissociation constants ( Matrix )
%   par   parameter vector for the free parameters
%   
%
% State Variables:
% [ketoglutarate_Matrix , COAS_Matrix , NAD_Matrix , CO2tot_Matrix , succinylcoA_Matrix , NADH_Matrix , ATP_Matrix , ADP_Matrix ]
%
%
% Free Parameters:
% [Vmf_AKGDH , Kir1 ]

function [f,J,varargout] = dXdT(t,x,T,BX,K_BX,par)
%% GLOBAL VARIABLES
% minimal parameter value
    MinCon = 0;


%% LIST OF STATE VARIABLES
% 1 ketoglutarate_Matrix
% 2 COAS_Matrix
% 3 NAD_Matrix
% 4 CO2tot_Matrix
% 5 succinylcoA_Matrix
% 6 NADH_Matrix
% 7 ATP_Matrix
% 8 ADP_Matrix
% 9 h_Matrix
% 10 m_Matrix
% 11 k_Matrix

% PARTIAL VOLUME FRACTIONS
VWater_Matrix = 1; % [=] l water (l region)^{-1}, Vinnakota and Bassingthwaighte, AJP, 2004 
VRegion_Matrix = 1; % [=] l region (l tissue)^{-1}, Vinnakota and Bassingthwaighte, AJP, 2004 

%% THERMODYNAMIC DATA
RT = 8.314*(T+273.15)/1e3; % kJ  mol^{-1}
F = 0.096484; % kJ mol^{-1} mV^{-1}

%% STATE VARIABLES

% Concentrations of Reference Species
ketoglutarate_Matrix = x(1);
COAS_Matrix = x(2);
NAD_Matrix = x(3);
CO2tot_Matrix = x(4);
succinylcoA_Matrix = x(5);
NADH_Matrix = x(6);
ATP_Matrix = x(7);
ADP_Matrix = x(8);
% Concentrations of H, Mg, and K
h_Matrix = x(9);
m_Matrix = x(10);
k_Matrix = x(11);

% Membrane potentials

%% DISSOCIATION CONSTANTS
% ketoglutarate_Matrix
Kh(1) = Inf;
Km(1) = Inf;
Kk(1) = Inf;
% COAS_Matrix
Kh(2) = 7.5727660576433292701044864528477085219293485351955e-09;
Km(2) = Inf;
Kk(2) = Inf;
% NAD_Matrix
Kh(3) = Inf;
Km(3) = Inf;
Kk(3) = Inf;
% CO2tot_Matrix
Kh(4) = 1.5083741503259146546832528365555438354395079159076e-10;
Km(4) = Inf;
Kk(4) = Inf;
% succinylcoA_Matrix
Kh(5) = 0.00010945977234272518557357811719299434116692282259464;
Km(5) = Inf;
Kk(5) = Inf;
% NADH_Matrix
Kh(6) = Inf;
Km(6) = Inf;
Kk(6) = Inf;
% ATP_Matrix
Kh(7) = 2.7990983755242071437057782383461823627612830023281e-07;
Km(7) = 0.00010815244062499600717370401570249782707833219319582;
Kk(7) = 0.097055055483606045285682739631738513708114624023438;
% ADP_Matrix
Kh(8) = 4.1856568564882886504828268456335216285424394300207e-07;
Km(8) = 0.0008821191357550335187392986924237447965424507856369;
Kk(8) = 0.13114858875318427533507303905935259535908699035645;

%% BINDING POLYNOMIALS
P( 1 ) = 1  + h_Matrix/Kh(1) + m_Matrix/Km(1) + k_Matrix/Kk(1);
P( 2 ) = 1  + h_Matrix/Kh(2) + m_Matrix/Km(2) + k_Matrix/Kk(2);
P( 3 ) = 1  + h_Matrix/Kh(3) + m_Matrix/Km(3) + k_Matrix/Kk(3);
P( 4 ) = 1  + h_Matrix/Kh(4) + m_Matrix/Km(4) + k_Matrix/Kk(4);
P( 5 ) = 1  + h_Matrix/Kh(5) + m_Matrix/Km(5) + k_Matrix/Kk(5);
P( 6 ) = 1  + h_Matrix/Kh(6) + m_Matrix/Km(6) + k_Matrix/Kk(6);
P( 7 ) = 1  + h_Matrix/Kh(7) + m_Matrix/Km(7) + k_Matrix/Kk(7);
P( 8 ) = 1  + h_Matrix/Kh(8) + m_Matrix/Km(8) + k_Matrix/Kk(8);

%% THERMODYNAMIC EQUATIONS
DGro_AKGDH =12.9535;

Keq_AKGDH_Matrix = exp(-DGro_AKGDH/RT)/P(1)/P(2)/P(3)*P(4)*P(5)*P(6)/h_Matrix;

%% FLUX EQUATIONS
%AKGDH_Matrix
a=ketoglutarate_Matrix;
b=COAS_Matrix;
c=NAD_Matrix;
p=CO2tot_Matrix;
q=succinylcoA_Matrix;
r=NADH_Matrix;
KmA=80e-6;
KmB=55e-6;
KmC=21e-6;
Kiq=6.9e-6;
Kir2=12.7e-6;
KiATP=50e-6;
KaADP=100e-6;
fATP=ATP_Matrix*(1+h_Matrix/Kh(7))/P(7);
fADP=ADP_Matrix*(1+h_Matrix/Kh(8))/P(8);
ai=1+KaADP/fADP*(1+fATP/KiATP);
Vmf_AKGDH=par(1);
Kir1=par(2);
Kir2=1e3;
if(a>MinCon)&&(b>MinCon)&&(c>MinCon)
J_AKGDH_Matrix=Vmf_AKGDH/(1+KmA/a*ai+KmB/b*(1+q/Kiq)+KmC/c*(1+r/Kir1))/(1+r/Kir2)*(1-1/Keq_AKGDH_Matrix*p*q*r/(a*b*c));
else
J_AKGDH_Matrix=0;
end

%% REACTANT TIME DERIVATIVES
f(1,:) = ( 0  - 1*J_AKGDH_Matrix ) / VWater_Matrix; % ketoglutarate_Matrix
f(2,:) = ( 0  - 1*J_AKGDH_Matrix ) / VWater_Matrix; % COAS_Matrix
f(3,:) = ( 0  - 1*J_AKGDH_Matrix ) / VWater_Matrix; % NAD_Matrix
f(4,:) = ( 0  + 1*J_AKGDH_Matrix ) / VWater_Matrix; % CO2tot_Matrix
f(5,:) = ( 0  + 1*J_AKGDH_Matrix ) / VWater_Matrix; % succinylcoA_Matrix
f(6,:) = ( 0  + 1*J_AKGDH_Matrix ) / VWater_Matrix; % NADH_Matrix
f(7,:) = ( 0  ); % [clamped] % ATP_Matrix
f(8,:) = ( 0  ); % [clamped] % ADP_Matrix

%% ION EQUATIONS
% COMPARTMENT Matrix:
ii = [1  2  3  4  5  6  7  8]; % Indices of SVs in compartment Matrix
% PARTIAL DERIVATIVES
pHBpM = -sum( (h_Matrix*x(ii)'./Kh(ii))./(Km(ii).*P(ii).^2) );
pHBpK = -sum( (h_Matrix*x(ii)'./Kh(ii))./(Kk(ii).*P(ii).^2) );
pHBpH = +sum( (1+m_Matrix./Km(ii)+k_Matrix./Kk(ii)).*x(ii)'./(Kh(ii).*P(ii).^2) );
pMBpH = -sum( (m_Matrix*x(ii)'./Km(ii))./(Kh(ii).*P(ii).^2) );
pMBpK = -sum( (m_Matrix*x(ii)'./Km(ii))./(Kk(ii).*P(ii).^2) );
pMBpM = +sum( (1+h_Matrix./Kh(ii)+k_Matrix./Kk(ii)).*x(ii)'./(Km(ii).*P(ii).^2) );
pKBpH = -sum( (k_Matrix*x(ii)'./Kk(ii))./(Kh(ii).*P(ii).^2) );
pKBpM = -sum( (k_Matrix*x(ii)'./Kk(ii))./(Km(ii).*P(ii).^2) );
pKBpK = +sum( (1+h_Matrix./Kh(ii)+m_Matrix./Km(ii)).*x(ii)'./(Kk(ii).*P(ii).^2) );
% PHIs
J_H = (0 - 1*J_AKGDH_Matrix) / VWater_Matrix;
J_M = (0) / VWater_Matrix;
J_K = (0) / VWater_Matrix;
Phi_H = J_H - sum( h_Matrix*f(ii)'./(Kh(ii).*P(ii)) );
Phi_M = -sum( m_Matrix*f(ii)'./(Km(ii).*P(ii)) );
Phi_K = J_K -sum( k_Matrix*f(ii)'./(Kk(ii).*P(ii)) );
% ALPHAs
aH = 1 + pHBpH;
aM = 1 + pMBpM;
aK = 1 + pKBpK;
% ADDITIONAL BUFFER for [H+]
aH = 1 + pHBpH + BX(1)/K_BX(1)/(1+h_Matrix/K_BX(1))^2; % M
% DENOMINATOR
D = aH*pKBpM*pMBpK + aK*pHBpM*pMBpH + aM*pHBpK*pKBpH - ...
    aM*aK*aH - pHBpK*pKBpM*pMBpH - pHBpM*pMBpK*pKBpH;
% DERIVATIVES for H,Mg,K
f(9,:) =  ( (pKBpM*pMBpK - aM*aK)*Phi_H + ...
            (aK*pHBpM - pHBpK*pKBpM)*Phi_M + ...
            (aM*pHBpK - pHBpM*pMBpK)*Phi_K ) / D;
f(10,:) =  ( (aK*pMBpH - pKBpH*pMBpK)*Phi_H + ...
            (pKBpH*pHBpK - aH*aK)*Phi_M + ...
            (aH*pMBpK - pHBpK*pMBpH)*Phi_K ) / D;
f(11,:) =  ( (aM*pKBpH - pKBpM*pMBpH)*Phi_H + ...
            (aH*pKBpM - pKBpH*pHBpM)*Phi_M + ...
            (pMBpH*pHBpM - aH*aM)*Phi_K ) / D;

%% ELECTROPHYS EQUATIONS
%% FLUX VECTOR:
J = [ J_AKGDH_Matrix];
